Contents:
---------

.. toctree::
   :maxdepth: 2

   whatsnew
   overview
   quickstart
   compatible_idioms
   imports
   what_else
   automatic_conversion
   faq
   stdlib_incompatibilities
   older_interfaces
   changelog
   credits
   reference

Indices and tables
------------------

* :ref:`genindex`
* :ref:`modindex`
* :ref:`search`

